@@ -1,4 +1,6 @@
 import pytest
+from sqlite_utils import Database
+from sqlite_utils.utils import sqlite3
 
 search_records = [
     {
@@ -173,3 +175,45 @@ def test_disable_fts(fresh_db, create_triggers):
         ).fetchone()[0]
     )
     assert ["searchable"] == fresh_db.table_names()
+
+
+@pytest.mark.parametrize("table_to_fix", ["searchable", "searchable_fts"])
+def test_rebuild_fts(fresh_db, table_to_fix):
+    table = fresh_db["searchable"]
+    table.insert(search_records[0])
+    table.enable_fts(["text", "country"])
+    # Run a search
+    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
+    # Delete from searchable_fts_data
+    fresh_db["searchable_fts_data"].delete_where()
+    # This should have broken the index
+    with pytest.raises(sqlite3.DatabaseError):
+        table.search("tanuki")
+    # Running rebuild_fts() should fix it
+    fresh_db[table_to_fix].rebuild_fts()
+    assert [("tanuki are running tricksters", "Japan", "foo")] == table.search("tanuki")
+
+
+@pytest.mark.parametrize("invalid_table", ["does_not_exist", "not_searchable"])
+def test_rebuild_fts_invalid(fresh_db, invalid_table):
+    fresh_db["not_searchable"].insert({"foo": "bar"})
+    # Raise OperationalError on invalid table
+    with pytest.raises(sqlite3.OperationalError):
+        fresh_db[invalid_table].rebuild_fts()
+
+
+@pytest.mark.parametrize("fts_version", ["FTS4", "FTS5"])
+def test_rebuild_removes_junk_docsize_rows(tmpdir, fts_version):
+    # Recreating https://github.com/simonw/sqlite-utils/issues/149
+    path = tmpdir / "test.db"
+    db = Database(str(path), recursive_triggers=False)
+    licenses = [{"key": "apache2", "name": "Apache 2"}, {"key": "bsd", "name": "BSD"}]
+    db["licenses"].insert_all(licenses, pk="key", replace=True)
+    db["licenses"].enable_fts(["name"], create_triggers=True, fts_version=fts_version)
+    assert db["licenses_fts_docsize"].count == 2
+    # Bug: insert with replace increases the number of rows in _docsize:
+    db["licenses"].insert_all(licenses, pk="key", replace=True)
+    assert db["licenses_fts_docsize"].count == 4
+    # rebuild should fix this:
+    db["licenses_fts"].rebuild_fts()
+    assert db["licenses_fts_docsize"].count == 2
